﻿Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.Drawing

Namespace RealTimeChessFrameWork


    ''' <summary>
    ''' clsCell
    ''' Pieceを置く1マスに対応
    ''' </summary>
    Friend Class clsCell
        Private _Piece As clsPiece
        Private _CellColor As CellColor
        Private _Grid As clsGrid
        Private _Position As Point
        Private _Rectangle As Rectangle


        ''' <summary>
        ''' 所持している駒
        ''' 何もないCellはNullとなる
        ''' </summary>
        Public Property Piece() As clsPiece
            Get
                Return _Piece
            End Get
            Set(ByVal value As clsPiece)
                _Piece = value
            End Set
        End Property



        ''' <summary>
        ''' Cellの色
        ''' 駒を持ち上げた時に置ける場所や取れる場所を色分けするためのもの
        ''' </summary>
        Public Property CellColor() As CellColor
            Get
                Return _CellColor
            End Get
            Set(ByVal value As CellColor)
                _CellColor = value
            End Set
        End Property



        ''' <summary>
        ''' 属するGrid
        ''' </summary>
        Public Property Grid() As clsGrid
            Get
                Return _Grid
            End Get
            Private Set(ByVal value As clsGrid)
                _Grid = value
            End Set
        End Property



        ''' <summary>
        ''' 倫理座標
        ''' 何個目のCellか
        ''' </summary>
        Public Property Position() As Point
            Get
                Return _Position
            End Get
            Private Set(ByVal value As Point)
                _Position = value
            End Set
        End Property



        ''' <summary>
        ''' 物理座標
        ''' Gridのどの場所にあるか
        ''' </summary>
        Public Property Rectangle() As Rectangle
            Get
                Return _Rectangle
            End Get
            Set(ByVal value As Rectangle)
                _Rectangle = value
            End Set
        End Property



        ''' <summary>
        ''' コンストラクタ
        ''' </summary>
        ''' <param name="g">属するGrid</param>
        ''' <param name="p">倫理座標</param>
        Public Sub New(ByVal g As clsGrid, ByVal p As Point)

            Me.Grid = g
            Me.Position = p

            Me.Piece = Nothing
            Me.CellColor = CellColor.None

            '倫理座標から物理座標を求める
            Me.Rectangle = New Rectangle(Me.Position.X * clsConst.CellSize.Width, _
                                         Me.Position.Y * clsConst.CellSize.Height, _
                                         clsConst.CellSize.Width, _
                                         clsConst.CellSize.Height)

        End Sub



        Public Sub Draw(ByVal g As Graphics)

            'Cellに色をつける
            Select Case Me.CellColor

                Case CellColor.None

                Case CellColor.Red
                    g.FillRectangle(New SolidBrush(Color.FromArgb(80, Color.Red)), Me.Rectangle)

                Case CellColor.Blue
                    g.FillRectangle(New SolidBrush(Color.FromArgb(80, Color.Blue)), Me.Rectangle)

            End Select

        End Sub


    End Class

End Namespace