﻿Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.Drawing
Imports System.Windows.Forms

Namespace RealTimeChessFrameWork

    ''' <summary>
    ''' 盤面の描画とCellの制御を行う
    ''' </summary>
    Friend Class clsGrid


        Private _cells As clsCell(,) = New clsCell(clsConst.XCount - 1, clsConst.YCount - 1) {}
        Private _ParentPictureBox As PictureBox
        Public ParentForm As frmMain


        Default Public Property Item(ByVal x As Integer, ByVal y As Integer) As clsCell
            Get
                Return _cells(x, y)
            End Get
            Set(ByVal value As clsCell)
                _cells(x, y) = value
            End Set
        End Property


        Public Property ParentPictureBox() As PictureBox
            Get
                Return _ParentPictureBox
            End Get
            Private Set(ByVal value As PictureBox)
                _ParentPictureBox = value
            End Set
        End Property


        ''' <summary>
        ''' コンストラクタ
        ''' </summary>
        ''' <param name="parent"></param>
        Public Sub New(ByVal parent As PictureBox)
            Me.ParentPictureBox = parent

            'Cellsの作成
            For x As Integer = 0 To clsConst.XCount - 1
                For y As Integer = 0 To clsConst.YCount - 1
                    Me(x, y) = New clsCell(Me, New Point(x, y))
                Next
            Next
        End Sub


        Public Sub AllCellPiecesClear()
            For Each c As clsCell In _cells
                If c.Piece IsNot Nothing AndAlso c.Piece.Parent IsNot Nothing Then
                    If c.Piece.Parent.Controls.Contains(c.Piece) Then
                        c.Piece.Parent.Controls.Remove(c.Piece)
                    End If
                End If
                c.Piece = Nothing
            Next
        End Sub


        Public Sub PutPiece(ByVal p As clsPiece)
            Me.ParentPictureBox.Controls.Add(p)
            Me(p.Place.X, p.Place.Y).Piece = p
        End Sub



        Public Sub MovePiece(ByVal p As clsPiece, ByVal after As Point)
            'セル移動したかどうか
            If p.Place <> after Then
                '移動していたら
                'もし敵がいたら
                If p.IsGettable(after) Then
                    Dim getPiece As clsPiece = p.Grid(after.X, after.Y).Piece
                    p.Parent.Controls.Remove(getPiece)
                    p.Owner.GetPiece(getPiece)
                    Me.OnGetPiece(getPiece)
                End If
                p.Grid(p.Place.X, p.Place.Y).Piece = Nothing
                'もといた場所を空に
                p.Place = after
                '移動
                p.Grid(p.Place.X, p.Place.Y).Piece = p
                p.IsMovable = False
                If p.Owner.IsMovable Then
                    '自分の駒なら
                    Me.OnMovePiece(p, p.Place)
                End If
            End If
            Me.ParentPictureBox.Refresh()
        End Sub



        Private Sub OnMovePiece(ByVal pi As clsPiece, ByVal po As Point)
            Dim s As String = Convert.ToString(pi.Idx) & "," & po.X & "," & po.Y
            Me.ParentForm.SendMessage(frmMain.enMsgKey.MOVE, s)
        End Sub



        Private Sub OnGetPiece(ByVal pi As clsPiece)
            Dim s As String = pi.Idx.ToString()
            Me.ParentForm.SendMessage(frmMain.enMsgKey.[GET], s)
            If pi.Design = Design.King Then
                Me.ParentForm.OwnWin = True
                Me.ParentForm.SendMessage(frmMain.enMsgKey.WIN, String.Empty)
            End If
        End Sub



        Public Sub AllCellClear()
            For x As Integer = 0 To clsConst.XCount - 1
                For y As Integer = 0 To clsConst.YCount - 1
                    Me(x, y).CellColor = CellColor.None
                Next
            Next
            Me.ParentPictureBox.Invalidate()
        End Sub



        Public Sub Draw(ByVal g As Graphics)
            Dim BorderPen As New Pen(Color.Beige, 3)

            For y As Integer = 0 To clsConst.YCount - 1
                For x As Integer = 0 To clsConst.XCount - 1
                    If (x + y) Mod 2 = 0 Then
                        g.FillRectangle(Brushes.Wheat, _
                                        x * clsConst.CellSize.Width, _
                                        y * clsConst.CellSize.Height, _
                                        clsConst.CellSize.Width, _
                                        clsConst.CellSize.Height)
                    End If
                    If (x + y) Mod 2 = 1 Then
                        g.FillRectangle(Brushes.Peru, _
                                        x * clsConst.CellSize.Width, _
                                        y * clsConst.CellSize.Height, _
                                        clsConst.CellSize.Width, _
                                        clsConst.CellSize.Height)
                    End If
                Next
            Next
            '左上
            g.DrawLine(BorderPen, 0, 0, clsConst.XCount * clsConst.CellSize.Width, 0)
            g.DrawLine(BorderPen, 0, 0, 0, clsConst.YCount * clsConst.CellSize.Height)
            '右下
            g.DrawLine(BorderPen, _
                       0, _
                       clsConst.YCount * clsConst.CellSize.Height - 1, _
                       clsConst.XCount * clsConst.CellSize.Width, _
                       clsConst.YCount * clsConst.CellSize.Height - 1)
            g.DrawLine(BorderPen, _
                       clsConst.XCount * clsConst.CellSize.Width - 1, _
                       0, _
                       clsConst.XCount * clsConst.CellSize.Width - 1, _
                       clsConst.YCount * clsConst.CellSize.Height)

            'Cell描画
            For x As Integer = 0 To clsConst.XCount - 1
                For y As Integer = 0 To clsConst.YCount - 1
                    Me(x, y).Draw(g)
                Next
            Next
        End Sub


    End Class


End Namespace