﻿Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.Drawing
Imports System.Windows.Forms

Namespace RealTimeChessFrameWork


    Friend Class clsPlayer



        Private _Name As String
        Private _IsMovable As Boolean
        Private _Ready As Boolean
        Private _Pieces As List(Of clsPiece)
        Private _GetPieces As List(Of clsPiece)
        Private _IsWinner As Boolean



        ''' <summary>
        ''' PlayerName
        ''' </summary>
        Public Property Name() As String
            Get
                Return _Name
            End Get
            Private Set(ByVal value As String)
                _Name = value
            End Set
        End Property



        ''' <summary>
        ''' 操作可能か
        ''' </summary>
        Public Property IsMovable() As Boolean
            Get
                Return _IsMovable
            End Get
            Private Set(ByVal value As Boolean)
                _IsMovable = value
            End Set
        End Property


        ''' <summary>
        ''' 準備完了
        ''' </summary>
        '''
        Public Property Ready() As Boolean
            Get
                Return _Ready
            End Get
            Set(ByVal value As Boolean)
                _Ready = value
            End Set
        End Property



        ''' <summary>
        ''' 所持ピース
        ''' 取られても減らない...
        ''' </summary>
        Public Property Pieces() As List(Of clsPiece)
            Get
                Return _Pieces
            End Get
            Private Set(ByVal value As List(Of clsPiece))
                _Pieces = value
            End Set
        End Property



        ''' <summary>
        ''' 取得したピース
        ''' </summary>
        Public Property GetPieces() As List(Of clsPiece)
            Get
                Return _GetPieces
            End Get
            Private Set(ByVal value As List(Of clsPiece))
                _GetPieces = value
            End Set
        End Property



        ''' <summary>
        ''' 所持ピースの数
        ''' </summary>
        Public ReadOnly Property PieceCount() As Integer
            Get
                Return Pieces.Count
            End Get
        End Property



        ''' <summary>
        ''' 取得したピースの数
        ''' </summary>
        Public ReadOnly Property GetPieceCount() As Integer
            Get
                Return GetPieces.Count
            End Get
        End Property



        ''' <summary>
        ''' 勝利
        ''' </summary>
        Public Property IsWinner() As Boolean
            Get
                Return _IsWinner
            End Get
            Set(ByVal value As Boolean)
                _IsWinner = value
            End Set
        End Property



        ''' <summary>
        ''' コンストラクタ
        ''' </summary>
        ''' <param name="name"></param>
        ''' <param name="isMovable"></param>
        Public Sub New(ByVal name As String, ByVal isMovable As Boolean)
            Me.Name = name
            Me.IsMovable = isMovable
            Me.Ready = False
        End Sub



        ''' <summary>
        ''' 持ち駒生成
        ''' </summary>
        ''' <param name="c"></param>
        ''' <param name="dir"></param>
        ''' <param name="g"></param>
        Public Sub NewPieces(ByVal c As PieceColor, ByVal dir As PieceDirection, ByVal g As clsGrid)
            Me.Pieces = Nothing
            Me.GetPieces = Nothing
            Me.Pieces = New List(Of clsPiece)()
            Me.GetPieces = New List(Of clsPiece)()

            Dim Row1 As Integer, Row2 As Integer
            If dir = PieceDirection.Up Then
                Row1 = 7
                Row2 = 6
                For i As Integer = 0 To 7
                    Me.Pieces.Add(New clsPiece(Design.Pawn, c, New Point(i, Row2), dir, g, Me))
                Next
                Me.Pieces.Add(New clsPiece(Design.Rook, c, New Point(0, Row1), dir, g, Me))
                Me.Pieces.Add(New clsPiece(Design.Knight, c, New Point(1, Row1), dir, g, Me))
                Me.Pieces.Add(New clsPiece(Design.Bishop, c, New Point(2, Row1), dir, g, Me))
                If c = PieceColor.Black Then
                    Me.Pieces.Add(New clsPiece(Design.King, c, New Point(3, Row1), dir, g, Me))
                    Me.Pieces.Add(New clsPiece(Design.Queen, c, New Point(4, Row1), dir, g, Me))
                Else
                    Me.Pieces.Add(New clsPiece(Design.Queen, c, New Point(3, Row1), dir, g, Me))
                    Me.Pieces.Add(New clsPiece(Design.King, c, New Point(4, Row1), dir, g, Me))
                End If
                Me.Pieces.Add(New clsPiece(Design.Bishop, c, New Point(5, Row1), dir, g, Me))
                Me.Pieces.Add(New clsPiece(Design.Knight, c, New Point(6, Row1), dir, g, Me))
                Me.Pieces.Add(New clsPiece(Design.Rook, c, New Point(7, Row1), dir, g, Me))
            Else
                Row1 = 0
                Row2 = 1
                For i As Integer = 7 To 0 Step -1
                    Me.Pieces.Add(New clsPiece(Design.Pawn, c, New Point(i, Row2), dir, g, Me))
                Next
                Me.Pieces.Add(New clsPiece(Design.Rook, c, New Point(7, Row1), dir, g, Me))
                Me.Pieces.Add(New clsPiece(Design.Knight, c, New Point(6, Row1), dir, g, Me))
                Me.Pieces.Add(New clsPiece(Design.Bishop, c, New Point(5, Row1), dir, g, Me))
                If c = PieceColor.Black Then
                    Me.Pieces.Add(New clsPiece(Design.King, c, New Point(4, Row1), dir, g, Me))
                    Me.Pieces.Add(New clsPiece(Design.Queen, c, New Point(3, Row1), dir, g, Me))
                Else
                    Me.Pieces.Add(New clsPiece(Design.Queen, c, New Point(4, Row1), dir, g, Me))
                    Me.Pieces.Add(New clsPiece(Design.King, c, New Point(3, Row1), dir, g, Me))
                End If
                Me.Pieces.Add(New clsPiece(Design.Bishop, c, New Point(2, Row1), dir, g, Me))
                Me.Pieces.Add(New clsPiece(Design.Knight, c, New Point(1, Row1), dir, g, Me))
                Me.Pieces.Add(New clsPiece(Design.Rook, c, New Point(0, Row1), dir, g, Me))
            End If

            '駒を持つ
            For i As Integer = 0 To Me.Pieces.Count - 1
                Pieces(i).Idx = i
            Next

        End Sub



        Public Sub PutPieces(ByVal g As clsGrid)
            For Each p As clsPiece In Pieces
                g.PutPiece(p)
            Next
        End Sub



        Public Sub GetPiece(ByVal p As clsPiece)
            Me.GetPieces.Add(p)
        End Sub



        '現在の駒の状況を描画
        Public Sub Draw(ByVal g As Graphics)
            If Pieces Is Nothing Then
                Return
            End If
            For Each p As clsPiece In Pieces
                p.Draw(g)
            Next
        End Sub
    End Class
End Namespace